unit ucuCadastrarLivro;


interface

uses
  utuStateMachine, utuRequest, ucuUseCase, acuFramework, nguLivro;

type
  ucCadastrarLivro = class(ucUseCase)
  public
    procedure Initialize; Override;
    procedure Finalize; Override;
  private
    fLivro: ngLivro;
    fSavePoint: acPersistenceSessionSavePoint;
  published
    procedure Effect_IncluirLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_EditarLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_BuscarLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ExcluirLivro(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ConcluirEdicao(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_Cancelar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_IncluirExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ExcluirExemplar(piRequest: utRequest; piTransition: utTransition);
  end;

implementation

uses
  ucuManager,
  utuMessage {define utField},
  utuEmprestimoOQL,
  acuObject {define acEnumerator},
  nguExemplar,

  nguSituacaoExemplar,

  nguEmprestimo,

  nguCliente,

  dateUtils;

procedure ucCadastrarLivro.Initialize;
begin
  inherited;
end;

procedure ucCadastrarLivro.Finalize;
begin
  
  inherited;
end;

procedure ucCadastrarLivro.Effect_IncluirLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro: utField;
begin
  fSavePoint := Self.Session.AddSavePoint;

  try
    fLivro := ngLivro.CreateNew(Self.Session);
    lFieldLivro := piRequest.Response.RootField.AddField('Livro');
    lFieldLivro.AddAttribute('OID').AsInteger := fLivro.IDO.AsVariant;

  except
    Self.Session.RollBackToSavePoint(fSavePoint);
    raise;
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_EditarLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro,
  lFieldExemplares,
  lFieldExemplar: utField;
  lExemplar: ngExemplar;
  lExemplares: acPersistentObjectList;
  lEnumExemplar: acEnumerator;
begin
  fSavePoint := Self.Session.AddSavePoint;

  lFieldLivro := piRequest.Message.RootField.FieldByName('Livro');

  {Tentamos carregar o livro}
  try
    fLivro := Self.Session.LoadObject(ngLivro,acOIDInt.Create(lFieldLivro.AttributeByName('OID').AsInteger)) as ngLivro;
  except
    on e:acObjectNotFoundException do
      raise EucException.Create('Livro a ser editado no pode ser encontrado.');
    else
      raise EucException.Create('No foi possvel editar este livro.');
   end;

   {Retornamos o livro e todos os seus exemplares}
   lExemplares := fLivro.Exemplares as acPersistentObjectList;

   lFieldLivro := piRequest.Response.RootField.AddField('Livro');
   fLivro.toXML(lFieldLivro);
   lFieldExemplares := lFieldLivro.AddField('Exemplares');

   lEnumExemplar := acEnumerator.Create(lExemplares);

   while not lEnumExemplar.EOL do
    begin
      lFieldExemplar := lFieldExemplares.AddField('Exemplar');
      lExemplar := lEnumExemplar.Current as ngExemplar;
      lExemplar.toXML(lFieldExemplar);
      
      lEnumExemplar.MoveNext;
    end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_BuscarLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivros,
  lFieldLivro: utField;
  lOQL: IObterLivros;
  lListLivros: acPersistentObjectList;
  lEnumLivro: acEnumerator;
  lLivro: ngLivro;
begin
  lFieldLivro:= piRequest.Message.RootField.FieldByName('Livro');

  lOQL := utEmprestimoOQL.ObterLivros(Self.Session);

  lFieldLivros := piRequest.Response.RootField.AddField('Livros');

  lListLivros := acPersistentObjectList.Create(ngLivro);
  try

    if lFieldLivro.HasAttribute('Titulo')
      then lOQL.Param_titulo.Value := lFieldLivro.AttributeByName('Titulo').AsString
      else lOQL.Param_titulo.SetNull;

    if lFieldLivro.HasAttribute('Autor')
      then lOQL.Param_autor.Value := lFieldLivro.AttributeByName('Autor').AsString
      else lOQL.Param_autor.SetNull;

    lOQL.LoadList(lListLivros);

    lEnumLivro := acEnumerator.Create(lListLivros);
    try
      while not lEnumLivro.EOL do
      begin
        lLivro := lEnumLivro.Current as ngLivro;
        lLivro.toXML(lFieldLivros.AddField('Livro'));

        lEnumLivro.MoveNext;
      end;
    finally
      lEnumLivro.Free;
    end;


  finally
    lListLivros.Free;
  end;

  piRequest.Processed := True;
end;

{Devido ao modelo, s ocorrer a excluso se o livro no possuir emprstimos.}
{Se ele possuir exemplares, eles sero deletados juntos com o livro.}
procedure ucCadastrarLivro.Effect_ExcluirLivro(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro,
  lFieldResult: utField;
  lLivro: ngLivro;
  lListExemplares: acPersistentObjectList;
  lOQL: IContarEmprestimosLivro;
  lSavePoint: acPersistenceSessionSavePoint;
begin

  lFieldLivro := piRequest.Message.RootField.FieldByName('Livro');
  lSavePoint := Session.AddSavePoint;
  
  try
    {Preparamos a busca OQL}
    lOQL := utEmprestimoOQL.ContarEmprestimosLivro(Self.Session);
    lOQL.Param_oid.Value := lFieldLivro.AttributeByName('OID').AsInteger;
    lFieldResult := utField.Create;

    try
      {Verificamos se ele tiver emprestimos}
      lOQL.LoadDataSet(lFieldResult);
       if lFieldResult.ValueByPath('DATAPACKET.ROWDATA.ROW.@C0').AsInteger > 0
        then raise EucException.Create('Nao foi possvel excluir o livro, ele possui pelo menos um exemplar com emprestimo associado.');

      {Deletamos o livro e seus exemplares, exemplares so deletado devido a efeito cascata da composio}
      lLivro := Self.Session.LoadObject(ngLivro,acOIDInt.Create(lFieldLivro.AttributeByName('OID').AsInteger)) as ngLivro;
      Self.Session.DeleteObject(lLivro);
      Self.Session.Save;
    finally
      lFieldResult.Free;
    end;
  except
    Self.Session.RollBackToSavePoint(lSavePoint);
    raise;
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_ConcluirEdicao(piRequest: utRequest; piTransition: utTransition);
var
  lFieldLivro,
  lFieldExemplares,
  lFieldExemplar: utField;
  lEnumExemplar: acEnumerator;
  lExemplares: acPersistentObjectList;
  lExemplar: ngExemplar;
  lSavePoint: acPersistenceSessionSavePoint;
begin

  lSavePoint := Self.Session.AddSavePoint;

  lFieldLivro := piRequest.Message.RootField.FieldByName('Livro');

  fLivro.Autor.Value := lFieldLivro.FindAttribute('Autor').AsString;
  fLivro.Titulo.Value := lFieldLivro.FindAttribute('Titulo').AsString;
  fLivro.Edicao.Value := lFieldLivro.FindAttribute('Edicao').AsString;

  Session.Save;

  lFieldLivro := piRequest.Response.RootField.AddField('Livro');
  fLivro.toXML(lFieldLivro);
  lFieldExemplares := lFieldLivro.AddField('Exemplares');

  lEnumExemplar := fLivro.Exemplares.GetEnumerator;
  try
    while not lEnumExemplar.EOL do
      begin
      lExemplar := lEnumExemplar.Current as ngExemplar;;
      lFieldExemplar := lFieldExemplares.AddField('Exemplar');
      lExemplar.toXML(lFieldExemplar);
      lEnumExemplar.MoveNext;
      end;
  finally
    lEnumExemplar.Free;
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_Cancelar(piRequest: utRequest; piTransition: utTransition);
begin
  if Assigned(fSavePoint)
    then Session.RollBackToSavePoint(fSavePoint);

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_IncluirExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar: utField;
  lExemplar: ngExemplar;
  lSavePoint: acPersistenceSessionSavePoint;
  lString: acString;
  lSituacao: ngSituacaoExemplar;
begin
  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');
  lSavePoint := Self.Session.AddSavePoint;

  try
    lExemplar :=  ngExemplar.CreateNew(Self.Session);
    lExemplar.Codigo.Value := lFieldExemplar.FindAttribute('Codigo').AsString;
    lExemplar.Livro.Partner := fLivro;

    try
      lSituacao := Self.Session.LoadObject(ngSituacaoExemplar,acOIDInt.Create(1)) as ngSituacaoExemplar;
    except
    on e:acObjectNotFoundException do
      raise EucException.Create('Situacao do exemplar no podeser encontrada.');
    else
      raise EucException.Create('No foi possvel atribuir uma situao a esse exemplar.');
    end;

    lExemplar.Situacao.Partner := lSituacao;
    
    lFieldExemplar := piRequest.Response.RootField.AddField('Exemplar');
    lExemplar.toXML(lFieldExemplar);
  except
    Self.Session.RollBackToSavePoint(lSavePoint);
    raise;
  end;

  piRequest.Processed := True;
end;

procedure ucCadastrarLivro.Effect_ExcluirExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar,
  lFieldResult: utField;
  lExemplar: ngExemplar;
  lSavePoint: acPersistenceSessionSavePoint;
  lOQL: IContarEmprestimosExemplar;
begin

  lSavePoint := Session.AddSavePoint;
  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');

  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');

  try
    {Verificamos se est na lista de exemplares do livro}
    lExemplar := Session.LoadObject(ngExemplar, acOIDInt.Create(lFieldExemplar.AttributeByName('OID').AsInteger)) as ngExemplar;
    if fLivro.Exemplares.IndexOf(lExemplar) = -1
      then raise EucException.Create('Exemplar nao esta na lista de exeplares deste livro.');

    {Preparamos a busca OQL}
    lOQL := utEmprestimoOQL.ContarEmprestimosExemplar(Self.Session);
    lOQL.Param_oid.Value := lFieldExemplar.AttributeByName('OID').AsInteger;

    lFieldResult := utField.Create;
    try
      {Verificamos se ele tiver emprestimos}
      lOQL.LoadDataSet(lFieldResult);
       if lFieldResult.ValueByPath('DATAPACKET.ROWDATA.ROW.@C0').AsInteger > 0
        then raise EucException.Create('Nao foi possvel excluir o exemplar, ele possui ao menos um emprestimo associado.');

      {Deletamos o exemplar de livro}
      lExemplar := Self.Session.LoadObject(ngExemplar,acOIDInt.Create(lFieldExemplar.AttributeByName('OID').AsInteger)) as ngExemplar;

      fLivro.Exemplares.Remove(lExemplar);
    finally
      lFieldResult.Free;
    end;
  except
    Self.Session.RollBackToSavePoint(lSavePoint);
    raise;
  end;

  piRequest.Processed := True;
end;


initialization

  gucManager.RegisterClass(3227,ucCadastrarLivro);

end.
